@@ -18,7 +18,10 @@ Example::
     demo()
     
 """
+
 from ipyautoui.autoui import AutoUi
 from ipyautoui.autodisplay import AutoDisplay
-from ipyautoui.autovjsf import AutoVjsf
+from ipyautoui.autovjsf import AutoVjsf  # TODO: make optional dependency...
 from ipyautoui.demo import demo
+
+__all__ = ["AutoUi", "AutoDisplay", "AutoVjsf", "demo"]
\ No newline at end of file
