@@ -1,25 +1,9 @@
-# -*- coding: utf-8 -*-
-# ---
-# jupyter:
-#   jupytext:
-#     formats: py:light
-#     text_representation:
-#       extension: .py
-#       format_name: light
-#       format_version: '1.5'
-#       jupytext_version: 1.11.5
-#   kernelspec:
-#     display_name: Python 3 (ipykernel)
-#     language: python
-#     name: python3
-# ---
-
 """
 displayfile is used to display certain types of files.
 The module lets us preview a data retrieved from: file, request or callable (< TODO)
 """
-# %run _dev_maplocal_params.py
-# %load_ext lab_black
+
+
 # +
 import os
 import pathlib
@@ -32,7 +16,7 @@ import ipywidgets as w
 import traitlets as tr
 import typing as ty
 from io import StringIO, BytesIO
-from pydantic import HttpUrl, parse_obj_as
+from pydantic import HttpUrl
 
 
 #  local imports
@@ -67,12 +51,14 @@ def getbytes(path: ty.Union[pathlib.Path, HttpUrl, ty.Callable]) -> ty.ByteStrin
     """common function for read bytes from: a request, file or callable
     NOTE: if a callable the data must be returned as bytes
     """
+
     def is_url(p):
-        try: 
+        try:
             HttpUrl(p)
             return True
         except:
             return False
+
     if isinstance(path, pathlib.Path):
         return path.read_bytes()
     elif is_url(path):
@@ -201,7 +187,7 @@ class PreviewExcel(w.VBox):
     @tr.observe("path")
     def _observe_path(self, change):
         byts = getbytes(self.path)
-        self.xl = pd.ExcelFile(BytesIO(byts))
+        self.xl = pd.ExcelFile(BytesIO(byts)) # NOTE: requires openpyxl
 
     @tr.observe("xl")
     def _observe_xl(self, change):
