@@ -1,45 +1,84 @@
-# ---
-# jupyter:
-#   jupytext:
-#     formats: py:light
-#     text_representation:
-#       extension: .py
-#       format_name: light
-#       format_version: '1.5'
-#       jupytext_version: 1.11.5
-#   kernelspec:
-#     display_name: Python 3 (ipykernel)
-#     language: python
-#     name: python3
-# ---
-
 # +
-# %run _dev_maplocal_params.py
-# %load_ext lab_black
+
 
 import typing as ty
 import ipywidgets as w
 from pydantic import BaseModel, Field
 from ipyautoui.nullable import nullable
-from ipyautoui._utils import frozenmap, obj_from_importstr
 from jsonref import replace_refs
 from ipyautoui.constants import MAP_JSONSCHEMA_TO_IPYWIDGET
-from ipyautoui._utils import remove_non_present_kwargs
+from ipyautoui._utils import remove_non_present_kwargs, frozenmap, obj_from_importstr
 from ipyautoui.custom.markdown_widget import MarkdownWidget
 from ipyautoui.custom.filechooser import FileChooser
 from ipyautoui.custom.date_string import DatePickerString, NaiveDatetimePickerString
 from ipyautoui.autobox import AutoBox
+from tempfile import TemporaryDirectory
+import pathlib
+from datamodel_code_generator import DataModelType, InputFileType, generate
+import json
+import importlib.util
+import sys
 
 import logging
 
 logger = logging.getLogger(__name__)
 
+def pydantic_model_file_from_json_schema(json_schema, fpth):
+    return generate(
+            json.dumps(json_schema, ensure_ascii=False),
+            input_file_type=InputFileType.JsonSchema,
+            input_filename="example.json",
+            output=fpth,
+            output_model_type=DataModelType.PydanticV2BaseModel,
+            capitalise_enum_members=True,
+        )
+
+def pydantic_model_from_json_schema(json_schema: dict) -> ty.Type[BaseModel]:
+    load = json_schema["title"].replace(" ", "") if "title" in json_schema else "Model"
+
+    with TemporaryDirectory() as temporary_directory_name:
+        temporary_directory = pathlib.Path(temporary_directory_name)
+        file_path = "model.py"
+        module_name = file_path.split(".")[0]
+        output = pathlib.Path(temporary_directory / file_path)
+        
+        pydantic_model_file_from_json_schema(json_schema, output)
+
+        #HACK refer to https://github.com/koxudaxi/datamodel-code-generator/issues/2534 for official fix, then remove the PATCH LOGIC once that is resolved
+        # --- NEW PATCH LOGIC ---
+        if json_schema.get("title") == "Project Building Area":
+            text = output.read_text()
+
+            # Replace Enum → IntEnum in TargetYear only
+            text = text.replace("class TargetYear(Enum):", "class TargetYear(IntEnum):")
+
+            # Ensure IntEnum is imported
+            if "from enum import IntEnum" not in text:
+                text = text.replace("from enum import Enum", "from enum import Enum, IntEnum")
+
+            output.write_text(text)
+        # --- END PATCH LOGIC ---
+
+        spec = importlib.util.spec_from_file_location(module_name, output)
+        module = importlib.util.module_from_spec(spec)
+        sys.modules[module_name] = module
+        spec.loader.exec_module(module)
+    return getattr(module, load)
 
 def _init_model_schema(
-    schema, by_alias=False
+    schema=None, by_alias=False, generate_pydantic_model_from_json_schema = True
 ) -> tuple[ty.Optional[ty.Type[BaseModel]], dict]:
+    if schema is None:
+        return None, {
+            "format": "dataframe",
+            "type": "array",
+            "items": {"properties": {}},
+        }
     if isinstance(schema, dict):
-        model = None  # jsonschema_to_pydantic(schema)
+        if generate_pydantic_model_from_json_schema:
+            model = pydantic_model_from_json_schema(schema)
+        else:
+            model = None
         # IDEA: Possible implementations -@jovyan at 8/24/2022, 12:05:02 PM
         # jsonschema_to_pydantic
         # https://koxudaxi.github.io/datamodel-code-generator/using_as_module/
@@ -47,54 +86,15 @@ def _init_model_schema(
         model = schema  # the "model" passed is a pydantic model
         schema = model.model_json_schema(by_alias=by_alias).copy()
 
-    schema = replace_refs(schema)
+    schema = replace_refs(schema, merge_props=True)
     schema = {k: v for k, v in schema.items() if k != "$defs"}
     return model, schema
 
+
 def pydantic_validate(model, value):
     return model.model_validate(value).model_dump(mode="json")
 
 
-# def is_Nullable(di: dict) -> bool:
-#     """
-#     is_Nullable({'title': 'floater', 'default': 1.33, 'anyOf': [{'type': 'string'}, {'type': 'null'}]})
-#     True
-#     """
-#     if "type" not in di.keys() and "anyOf" in di.keys():
-#         if "null" in [l.get("type") for l in di["anyOf"]]:
-#             return True
-#         else:
-#             return False
-#     else:
-#         return False
-
-
-# def get_type(di: dict) -> str:
-#     if "type" in di.keys():
-#         return di["type"]
-#     else:
-#         if "anyOf" in di.keys():
-#             kw = "anyOf"
-#         elif "allOf" in di.keys():
-#             kw = "allOf"
-#         else:
-#             raise ValueError("currently must have anyOf or allOf or type in schema")
-#         types = list(set([l.get("type") for l in di[kw]]))
-#         if len(types) > 2:
-#             raise ValueError(
-#                 f"currently a single type must be specified. a specific type + null is also allowed. not {str([l['type'] for l in di['anyOf']])}"
-#             )
-#         else:
-#             t = [l for l in types if l != "null"][0]
-#             return t
-
-
-# def get_type_and_nullable(di: dict) -> tuple[str, bool]:
-#     type_ = get_type(di)
-#     nullable = is_Nullable(di)
-#     return type_, nullable
-
-
 def is_allowed_type(di: dict) -> bool:
     #  https://json-schema.org/understanding-json-schema/reference/combining.html
     if "anyOf" in di:
@@ -118,7 +118,7 @@ def flatten_allOf(di: dict) -> dict:
     if "allOf" in di.keys():
         for _ in di["allOf"]:
             di = {**di, **_}
-        return {k:v for k,v in di.items() if k != "allOf"}
+        return {k: v for k, v in di.items() if k != "allOf"}
     else:
         return di
 
@@ -740,10 +740,11 @@ def is_Textarea(
         return True, allow_none
     else:
         return False, allow_none
-    
+
+
 def is_enum_array(di: dict) -> (bool, int):
     is_enum = lambda di: True if "enum" in di.keys() else False
-    
+
     if not "items" in di.keys():
         return False, 0
     if is_enum(di["items"]):
@@ -757,8 +758,9 @@ def is_enum_array(di: dict) -> (bool, int):
                 return False, 0
         else:
             return False, 0
-        
-def is_select_multiple(di:dict) ->( bool, int):
+
+
+def is_select_multiple(di: dict) -> (bool, int):
     t = di["type"]
     if "autoui" in di.keys():
         return False, 0
@@ -768,8 +770,10 @@ def is_select_multiple(di:dict) ->( bool, int):
         return False, 0
     return is_enum_array(di)
 
+
 SELECT_MULTIPLE_MAX_ITEMS = 10
 
+
 def is_SelectMultiple(
     di: dict, allow_none=False, checked_nullable=False
 ) -> tuple[bool, bool]:
@@ -794,7 +798,8 @@ def is_SelectMultiple(
             return True, allow_none
         else:
             return False, allow_none
-    
+
+
 def is_TagsInput(
     di: dict, allow_none=False, checked_nullable=False
 ) -> tuple[bool, bool]:
@@ -933,7 +938,9 @@ def remove_title_and_description(di):
     return {k: v for k, v in di.items() if k != "description" and k != "title"}
 
 
-def create_widget_caller(schema, calling=None, remove_title=True, map_keys=MAP_JSONSCHEMA_TO_IPYWIDGET):
+def create_widget_caller(
+    schema, calling=None, remove_title=True, map_keys=MAP_JSONSCHEMA_TO_IPYWIDGET
+):
     """
     creates a "caller" object from the schema.
     this renames schema keys as follows to match ipywidgets:
@@ -966,13 +973,18 @@ def create_widget_caller(schema, calling=None, remove_title=True, map_keys=MAP_J
         caller = remove_non_present_kwargs(calling, caller)
     return caller
 
+
 def tags_widget_caller(schema, calling=None, remove_title=True):
     map_keys = dict(MAP_JSONSCHEMA_TO_IPYWIDGET) | {"enum": "allowed_tags"}
-    return create_widget_caller(schema, calling=calling, remove_title=remove_title, map_keys=map_keys)
+    return create_widget_caller(
+        schema, calling=calling, remove_title=remove_title, map_keys=map_keys
+    )
+
 
 def flatten_items(di: dict, fn=None) -> dict:
-    di = {**di, **{k:v for k, v in di["items"].items() if k!="type"}}
-    return {k:v for k,v in di.items() if k!="items"}
+    di = {**di, **{k: v for k, v in di["items"].items() if k != "type"}}
+    return {k: v for k, v in di.items() if k != "items"}
+
 
 def flatten_type_and_nullable(di: dict, fn=None) -> dict:
     if "type" in di.keys():
@@ -986,7 +998,9 @@ def flatten_type_and_nullable(di: dict, fn=None) -> dict:
             raise ValueError("currently must have anyOf or allOf or type in schema")
         types = di[kw]
         is_nullable = "null" in [l.get("type") for l in types]
-        types_non_null = [l for l in types if l.get("type") != "null"]  # get non-null types
+        types_non_null = [
+            l for l in types if l.get("type") != "null"
+        ]  # get non-null types
 
         def get_enum(types_non_null):
             li = [_["enum"] for _ in types_non_null if "enum" in _.keys()]
@@ -999,7 +1013,7 @@ def flatten_type_and_nullable(di: dict, fn=None) -> dict:
         if len(types_non_null) > 1 and enum is not None:
             di["examples"] = enum
         for _ in types_non_null:
-            di = {**di, **_}
+            di = {**_, **di}
         del di[kw]
         return {**di, **{"nullable": is_nullable}}
 
@@ -1110,11 +1124,32 @@ schema:
 from ipyautoui.autoanyof import AnyOf
 
 
-def get_widgets_map(di_update=None):
+def get_containers_map(di_update=None):
     from ipyautoui.custom.iterable import AutoArray
     from ipyautoui.autoobject import AutoObject
     from ipyautoui.custom.editgrid import EditGrid
 
+    CONTAINERS_MAP = frozenmap(
+        **{
+            "object": WidgetMapper(fn_filt=is_Object, widget=AutoObject),
+            "array": WidgetMapper(
+                fn_filt=is_Array,
+                widget=AutoArray,
+                li_fn_modify=[flatten_type_and_nullable],
+            ),
+            "dataframe": WidgetMapper(
+                fn_filt=is_DataFrame,
+                widget=EditGrid,
+                li_fn_modify=[add_schema_key, add_max_layout],
+            ),
+        }
+    )
+    if di_update is None:
+        di_update = {}
+    return update_widgets_map(CONTAINERS_MAP, di_update=di_update)
+
+
+def get_widgets_map(di_update=None):
     WIDGETS_MAP = frozenmap(
         **{
             "AutoOveride": WidgetMapper(
@@ -1216,23 +1251,13 @@ def get_widgets_map(di_update=None):
             "Datetime": WidgetMapper(
                 fn_filt=is_Datetime, widget=NaiveDatetimePickerString
             ),
-            "object": WidgetMapper(fn_filt=is_Object, widget=AutoObject),
-            "array": WidgetMapper(
-                fn_filt=is_Array,
-                widget=AutoArray,
-                li_fn_modify=[flatten_type_and_nullable],
-            ),
-            "dataframe": WidgetMapper(
-                fn_filt=is_DataFrame,
-                widget=EditGrid,
-                li_fn_modify=[add_schema_key, add_max_layout],
-            ),
             "anyOf": WidgetMapper(
                 fn_filt=is_AnyOf,
                 widget=AnyOf,
                 li_fn_modify=[create_widget_caller],
             ),
         }
+        | dict(get_containers_map())
     )
 
     if di_update is None:
@@ -1292,10 +1317,13 @@ def map_widget(
         widget_name, allow_none = mapped[0]
         wi = get_widget(di, widget_name, widgets_map)
         kwargs = di
-        for l in widgets_map[widget_name].li_fn_modify:
-            kwargs = l(kwargs, wi)
+        for x in widgets_map[widget_name].li_fn_modify:
+            kwargs = x(kwargs, wi)
 
-        kwargs_box = update_keys(di)
+        kwargs_box = flatten_type_and_nullable(di)
+        # TODO: ^ probs a more efficient way to do this to avoid repetition with above
+        #          as `flatten_type_and_nullable` is typically in `li_fn_modify`
+        kwargs_box = update_keys(kwargs_box)
         kwargs_box = remove_non_present_kwargs(AutoBox, kwargs_box)
         return WidgetCaller(
             schema_=di,
@@ -1323,23 +1351,12 @@ def get_widget(di, **kwargs):
     return widgetcaller(caller)  # TODO: add passing of value
 
 
-if __name__ == "__main__":
-    di = {
-        "title": "int",
-        "default": 3,
-        "anyOf": [{"type": "integer"}, {"type": "null"}],
-    }
-    caller = map_widget(di)
-    assert "IntText" in str(caller.autoui)
-    print(caller)
-
-
 def from_schema_method(
     cls, schema: ty.Union[ty.Type[BaseModel], dict], value: ty.Optional[dict] = None
 ):
     if "$defs" in schema.keys():
         try:
-            schema = replace_refs(schema)
+            schema = replace_refs(schema, merge_props=True)
         except ValueError as e:
             logger.warning(f"replace_refs error: \n{e}")
             pass
@@ -1349,9 +1366,20 @@ def from_schema_method(
 
 
 def from_model_method(cls, model: ty.Type[BaseModel], value: ty.Optional[dict] = None):
-    schema = replace_refs(model.model_json_schema())
+    schema = replace_refs(model.model_json_schema(), merge_props=True)
     if value is not None:
         schema["value"] = value
     ui = cls(**schema)
     ui.model = model
     return ui
+
+
+if __name__ == "__main__":
+    di = {
+        "title": "int",
+        "default": 3,
+        "anyOf": [{"type": "integer"}, {"type": "null"}],
+    }
+    caller = map_widget(di)
+    assert "IntText" in str(caller.autoui)
+    print(caller)
