@@ -1,33 +1,19 @@
-# -*- coding: utf-8 -*-
-# ---
-# jupyter:
-#   jupytext:
-#     cell_metadata_filter: -all
-#     formats: py:light
-#     text_representation:
-#       extension: .py
-#       format_name: light
-#       format_version: '1.5'
-#       jupytext_version: 1.14.0
-#   kernelspec:
-#     display_name: Python 3 (ipykernel)
-#     language: python
-#     name: python3
-# ---
-
 # +
 """
 a UI element that loads a folder for data caching, whilst storing a record of folders in use
 """
 
-# %run ../_dev_maplocal_params.py
-#
-# %load_ext lab_black
+
+# -
+
 from pydantic import ConfigDict, BaseModel
 import ipywidgets as w
 import typing as ty
 import traitlets as tr
 
+# +
+TreeModel = ty.ForwardRef("TreeModel")
+
 
 class TreeModel(BaseModel):
     """generic tree model."""
@@ -36,15 +22,12 @@ class TreeModel(BaseModel):
     description: str = ""
     options: list
     value: ty.Union[str, float, int] = None  # ty.Union[str, float, int]
-    children: ty.ForwardRef("TreeModel") = None
+    children: TreeModel = None
     disabled: bool = False
     placeholder: str = ""
     model_config = ConfigDict(extra="allow")
 
 
-TreeModel.update_forward_refs()
-
-
 # +
 def gen_widget(di, widgets=[]):
     wi = w.Combobox(**di, ensure_option=True, layout={"width": "200px"})
@@ -109,6 +92,7 @@ class DecisionUi(w.HBox):
 
 if __name__ == "__main__":
     from IPython.display import display
+
     PROJECTS = ["J5001", "J5002"]
     t = TreeModel(
         **{
