@@ -1,24 +1,7 @@
-# -*- coding: utf-8 -*-
-# ---
-# jupyter:
-#   jupytext:
-#     cell_metadata_filter: -all
-#     formats: py:light
-#     text_representation:
-#       extension: .py
-#       format_name: light
-#       format_version: '1.5'
-#       jupytext_version: 1.15.2
-#   kernelspec:
-#     display_name: Python 3 (ipykernel)
-#     language: python
-#     name: python3
-# ---
-
 # +
 """General widget for editing data"""
-# %run ../_dev_maplocal_params.py
-# %load_ext lab_black
+
+
 # TODO: move editgrid.py to root ?
 
 import traitlets as tr
@@ -28,17 +11,19 @@ import traceback
 import pandas as pd
 import ipywidgets as w
 from IPython.display import clear_output, display
-from markdown import markdown
-from pydantic import BaseModel, Field
+from pydantic import BaseModel, RootModel, Field
+import json
 
 from ipyautoui.autoobject import AutoObjectForm
 from ipyautoui.custom.buttonbars import CrudButtonBar
-from ipyautoui._utils import frozenmap
+from ipyautoui._utils import frozenmap, traits_in_kwargs
 from ipyautoui.constants import BUTTON_WIDTH_MIN
 from ipyautoui.custom.autogrid import AutoGrid
+from ipyautoui.custom.edittsv import EditTsvWithDiff
 from ipyautoui.custom.title_description import TitleDescription
 
 MAP_TRANSPOSED_SELECTION_MODE = frozenmap({True: "column", False: "row"})
+logger = logging.getLogger(__name__)
 # TODO: rename "add" to "fn_add" so not ambiguous...
 # -
 
@@ -59,26 +44,18 @@ class DataHandler(BaseModel):
 
     # REVIEW... MAYBE SHOULD USE *ARGS AND **KWARGS
     fn_get_all_data: ty.Callable  # TODO: rename to fn_get
-    fn_post: ty.Callable[[dict], None]
+    fn_post: ty.Callable[[dict], None] # should return int
     fn_patch: ty.Callable[[ty.Any, dict], None]  # TODO: need to add index
     fn_delete: ty.Callable[[list[int]], None]
     fn_copy: ty.Callable[[list[int]], None]
+    fn_io: ty.Optional[ty.Callable] = None
 
 
 if __name__ == "__main__":
+    from ipyautoui.demo_schemas import EditableGrid
+    from ipyautoui.demo_schemas.editable_datagrid import DataFrameCols
 
-    class TestModel(BaseModel):
-        string: str = Field("string", title="Important String")
-        integer: int = Field(40, title="Integer of somesort")
-        floater: float = Field(1.33, title="floater")
-
-    def test_save():
-        print("Saved.")
-
-    def test_revert():
-        print("Reverted.")
-
-    ui = AutoObjectForm.from_pydantic_model(TestModel)
+    ui = AutoObjectForm.from_pydantic_model(DataFrameCols)
     display(ui)
 
 if __name__ == "__main__":
@@ -89,16 +66,7 @@ if __name__ == "__main__":
     ui.value = {"string": "adfs", "integer": 2, "floater": 1.22}
 
 
-class RowEditor:
-    fn_add: ty.List[ty.Callable[[ty.Any, dict], None]]  # post
-    fn_edit: ty.List[ty.Callable[[ty.Any, dict], None]]  # patch
-    fn_move: ty.Callable
-    fn_copy: ty.Callable
-    fn_delete: ty.Callable
-
-
-# +
-class UiDelete(w.HBox):
+class UiDelete(w.VBox):
     value = tr.Dict(default_value={})
     columns = tr.List(default_value=[])
 
@@ -134,9 +102,9 @@ class UiDelete(w.HBox):
         self.fn_delete = fn_delete
         self.out_delete = w.Output()
         self.bn_delete = w.Button(
-            icon="trash",
+            description="DELETE",
             button_style="danger",
-            layout=w.Layout(width=BUTTON_WIDTH_MIN),
+            layout=w.Layout(width="100px"),
         )
         self.vbx_messages = w.VBox()
         self.message = w.HTML(
@@ -234,13 +202,13 @@ if __name__ == "__main__":
 # TODO: refactor how the datahandler works...
 # TODO: add a test for the datahandler...
 
-
 # from ipyautoui.watch_validate import WatchValidate
 class EditGrid(w.VBox, TitleDescription):
     _value = tr.Tuple()  # using a tuple to guarantee no accidental mutation
     warn_on_delete = tr.Bool()
     show_copy_dialogue = tr.Bool()
     close_crud_dialogue_on_action = tr.Bool()
+    show_ui_io = tr.Bool(default_value=False)
 
     @tr.observe("warn_on_delete")
     def observe_warn_on_delete(self, on_change):
@@ -255,6 +223,23 @@ class EditGrid(w.VBox, TitleDescription):
             self.ui_copy.layout.display = ""
         else:
             self.ui_copy.layout.display = "None"
+    
+    @property
+    def json(self):  # HOTFIX: not required if WatchValidate is used
+        return json.dumps(self.value, indent=4)
+
+    @property
+    def transposed(self):
+        return self.grid.transposed
+
+    @transposed.setter
+    def transposed(self, value: bool):
+        self.grid.transposed = value
+        if getattr(self, "ui_io", None) is not None:
+            if "transposed" in self.ui_io.traits():
+                self.ui_io.transposed = value
+            else:
+                logger.warning("transposed not found in ui_io")
 
     @property
     def value(self):
@@ -270,19 +255,25 @@ class EditGrid(w.VBox, TitleDescription):
         self.grid.transform(_transforms)  # Set to previous transforms
 
     @property
-    def transposed(self):
-        return self.grid.transposed
+    def schema(self):
+        return self.grid.schema
 
-    @transposed.setter
-    def transposed(self, value: bool):
-        self.grid.transposed = value
+    @property
+    def row_schema(self):
+        return self.grid.schema["items"]
 
-    def _update_value_from_grid(self):
-        self._value = self.grid.records()
+    @property
+    def model(self):
+        return self.grid.model
+
+    # TODO: this is a bit of a hack... need to refactor when EditGrid inherits WatchValidate
+    @classmethod
+    def from_pydantic_model(cls, model: ty.Type[BaseModel], **kwargs):
+        return cls(model, **kwargs)
 
     def __init__(
         self,
-        schema: ty.Union[dict, ty.Type[BaseModel]],
+        schema: ty.Optional[ty.Union[dict, ty.Type[BaseModel]]] = None,
         value: ty.Optional[list[dict[str, ty.Any]]] = None,
         by_alias: bool = False,
         by_title: bool = True,
@@ -291,30 +282,104 @@ class EditGrid(w.VBox, TitleDescription):
         ui_edit: ty.Optional[ty.Callable] = None,
         ui_delete: ty.Optional[ty.Callable] = None,
         ui_copy: ty.Optional[ty.Callable] = None,
+        ui_io: ty.Optional[ty.Callable] = None,
         warn_on_delete: bool = False,
         show_copy_dialogue: bool = False,
         close_crud_dialogue_on_action: bool = False,
         title: str = None,
         description: str = None,
         show_title: bool = True,
+        generate_pydantic_model_from_json_schema: bool = False,
+        show_ui_io: bool = False,
         **kwargs,
     ):  # TODO: use **kwargs to pass attributes to EditGrid as in AutoObject and AutoArray
-        self.html_title = w.HTML()
+        self.vbx_error = w.VBox()
+        self.vbx_widget = w.VBox(layout={"width": "100%"})
+        # TODO: ^ move common container attributes to WatchValidate
         self.description = description
         self.title = title
         self.show_title = show_title
-
         self.by_title = by_title
         self.by_alias = by_alias
         self.datahandler = datahandler
+        self.generate_pydantic_model_from_json_schema = generate_pydantic_model_from_json_schema
+
+        self.ui_io = None
+        self._ui_io_factory = None
+        self.show_ui_io = show_ui_io
+        self.ui_io_initialised = False
+
+        self.close_crud_dialogue_on_action = close_crud_dialogue_on_action
+        self._init_autogrid(schema, value, **kwargs)
+        self._init_ui_callables(
+            ui_add=ui_add, ui_edit=ui_edit, ui_delete=ui_delete, ui_copy=ui_copy, ui_io=ui_io
+        )
+        self._init_form()
+        self._init_row_controls()
+        self._init_controls()
+        super().__init__(
+            **{
+                k: v
+                for k, v in kwargs.items()
+                if k not in traits_in_kwargs(AutoGrid, kwargs)
+            }
+        )  # NOTE: Only pass kwargs not in AutoGrid traits
+        self.warn_on_delete = warn_on_delete
+        # self.show_copy_dialogue = show_copy_dialogue
+        self.show_copy_dialogue = False
+        # ^ TODO: delete this when that functionality is added
+        self._set_children_editgrid()
+        self._set_datahandler(datahandler=datahandler)
+        self._update_value_from_grid()
+
+    def update_from_schema(
+        self,
+        schema: ty.Optional[ty.Union[dict, ty.Type[BaseModel]]] = None,
+        value: ty.Optional[list[dict[str, ty.Any]]] = None,
+        datahandler: ty.Optional[DataHandler] = None,
+        ui_add: ty.Optional[ty.Callable] = None,
+        ui_edit: ty.Optional[ty.Callable] = None,
+        ui_delete: ty.Optional[ty.Callable] = None,
+        ui_copy: ty.Optional[ty.Callable] = None,
+        ui_io: ty.Optional[ty.Callable] = None,
+        **kwargs,
+    ):
+        getvalue = lambda value: (
+            None if value is None or value == [{}] else pd.DataFrame(value)
+        )
+        self.grid.update_from_schema(
+            schema, data=getvalue(value), by_alias=self.by_alias, generate_pydantic_model_from_json_schema=self.generate_pydantic_model_from_json_schema, **kwargs
+        )
+        self._init_ui_callables(
+            ui_add=ui_add, ui_edit=ui_edit, ui_delete=ui_delete, ui_copy=ui_copy, ui_io=ui_io
+        )
+        self._init_ui_io(ui_io=ui_io)
+        self._init_row_controls()
+        self._init_controls()
+        self._set_children_editgrid()
+        self._set_datahandler(datahandler=datahandler)
+
+    def _init_autogrid(
+        self,
+        schema: ty.Union[dict, ty.Type[BaseModel]],
+        value: ty.Optional[list[dict[str, ty.Any]]],
+        **kwargs,
+    ):
         getvalue = lambda value: (
             None if value is None or value == [{}] else pd.DataFrame(value)
         )
         self.grid = AutoGrid(
-            schema, data=getvalue(value), by_alias=self.by_alias, **kwargs
+            schema, data=getvalue(value), generate_pydantic_model_from_json_schema=self.generate_pydantic_model_from_json_schema, by_alias=self.by_alias, **kwargs
         )
 
-        self._init_form()
+    def _init_ui_callables(
+        self,
+        ui_add: ty.Optional[ty.Callable] = None,
+        ui_edit: ty.Optional[ty.Callable] = None,
+        ui_delete: ty.Optional[ty.Callable] = None,
+        ui_copy: ty.Optional[ty.Callable] = None,
+        ui_io: ty.Optional[ty.Callable] = None,
+    ):
         if ui_add is None:
             self.ui_add = AutoObjectForm.from_jsonschema(self.row_schema)
         else:
@@ -332,27 +397,60 @@ class EditGrid(w.VBox, TitleDescription):
             self.ui_copy = UiCopy()
         else:
             self.ui_copy = ui_copy()
+        self.ui_io = None
+        self._ui_io_factory = None
+        if self.show_ui_io:
+            def _missing_model_ui():
+                return w.HTML("must instantiate with pydantic model for this feature")
+
+            if ui_io is None:
+                def _factory():
+                    if self.model is None:
+                        return _missing_model_ui()
+                    return EditTsvWithDiff(
+                        model=self.model, fn_upload=self.fn_upload, transposed=self.transposed
+                    )
+                self._ui_io_factory = _factory
+            else:
+                def _factory_custom():
+                    if self.model is None:
+                        return _missing_model_ui()
+                    try:
+                        return ui_io(
+                            model=self.model, fn_upload=self.fn_upload, transposed=self.transposed
+                        )
+                    except Exception as e:
+                        raise RuntimeError(
+                            f"Failed to initialize ui_io '{ui_io.__name__}'."
+                            " Required traits are: `model`, `fn_upload`, `transposed`."
+                            f" Original error: {e}"
+                        ) from e
+                self._ui_io_factory = _factory_custom
         self.ui_copy.layout.display = "None"
-        self.warn_on_delete = warn_on_delete
-        # self.show_copy_dialogue = show_copy_dialogue
-        self.show_copy_dialogue = False
-        # ^ TODO: delete this when that functionality is added
-        self.close_crud_dialogue_on_action = close_crud_dialogue_on_action
         self.ui_delete.fn_delete = self._delete_selected
-        self._update_value_from_grid()
-        self._init_row_controls()
-        self.stk_crud = w.Stack(
-            children=[self.ui_add, self.ui_edit, self.ui_copy, self.ui_delete]
-        )
-        self.children = [
-            self.html_title,
-            self.buttonbar_grid,
-            self.stk_crud,
-            self.grid,
-        ]
-        self._init_controls()
-        if self.datahandler is not None:
-            self.buttonbar_grid.fn_reload = self._reload_datahandler
+
+    def _init_ui_io(self, ui_io):
+        if ui_io is not None and self.ui_io_initialised:
+            self.ui_io = ui_io(
+                model=self.model, fn_upload=self.fn_upload, transposed=self.transposed
+            )
+
+    def _ensure_ui_io_initialised(self):
+        if not self.show_ui_io:
+            return None
+        if self.ui_io is not None:
+            return self.ui_io
+        if self._ui_io_factory is None:
+            return None
+        self.ui_io = self._ui_io_factory()
+        if hasattr(self.ui_io, "traits") and "transposed" in self.ui_io.traits():
+            self.ui_io.transposed = self.transposed
+        self._set_children_editgrid()
+        return self.ui_io
+
+    def fn_upload(self, value):
+        """This method sets the grid's value to the ui_io value. Override this method to add additional functionality to save changes."""
+        self.value=value
 
     def _init_row_controls(self):
         self.ui_edit.show_savebuttonbar = True
@@ -362,20 +460,7 @@ class EditGrid(w.VBox, TitleDescription):
         self.ui_add.savebuttonbar.fns_onsave = [self._post, self._save_add_to_grid]
         self.ui_add.savebuttonbar.fns_onrevert = [self._set_ui_add_to_default_row]
 
-    @property
-    def schema(self):
-        return self.grid.schema
-
-    @property
-    def row_schema(self):
-        return self.grid.schema["items"]
-
-    @property
-    def model(self):
-        return self.grid.model
-
     def _init_form(self):
-        super().__init__()
         get_reload = lambda: (
             None if self.datahandler is None else self._reload_datahandler
         )
@@ -385,11 +470,12 @@ class EditGrid(w.VBox, TitleDescription):
             fn_copy=self._copy,
             fn_delete=self._delete,
             fn_reload=get_reload(),
-            # backward=self.setview_default,
-            show_message=False,
+            fn_io=self._io,
+            show_io=self.show_ui_io
+        )
+        self.stk_crud = w.Stack(
+            children=[self.ui_add, self.ui_edit, self.ui_copy, self.ui_delete]
         )
-        self.addrow = w.VBox()
-        self.editrow = w.VBox()
 
     def _init_controls(self):
         self.grid.observe(self._observe_selections, "selections")
@@ -398,6 +484,27 @@ class EditGrid(w.VBox, TitleDescription):
         self.grid.observe(self._observe_order, "order")
         self._observe_order(None)  # prompts order if it is set in by grid setter above
 
+    def _update_value_from_grid(self):
+        self._value = self.grid.records()
+
+    def _set_datahandler(self, datahandler):
+        self.datahandler = datahandler
+        if self.datahandler is not None:
+            self.buttonbar_grid.fn_reload = self._reload_datahandler
+
+    def _set_children_editgrid(self):
+        self.vbx_widget.children = [self.buttonbar_grid, self.stk_crud, self.grid]
+    
+        # Base CRUD UIs
+        children = [self.ui_add, self.ui_edit, self.ui_copy, self.ui_delete]
+    
+        # Conditionally add ui io
+        if self.show_ui_io and self.ui_io is not None:
+            children.append(self.ui_io)
+    
+        self.stk_crud.children = children
+        self.children = [self.hbx_title_description, self.vbx_widget]
+
     def _observe_order(self, on_change):
         if "order" in self.ui_add.traits() and self.grid.order is not None:
             self.ui_add.order = self.grid.order
@@ -418,16 +525,19 @@ class EditGrid(w.VBox, TitleDescription):
         self._update_value_from_grid()
 
     def _setview(self, onchange):
+        if self.buttonbar_grid.active == "io":
+            self.ui_io_initialised = True
+            self._ensure_ui_io_initialised()
         if self.buttonbar_grid.active is None:
             self.stk_crud.selected_index = None
+        elif self.buttonbar_grid.active == "support":
+            self.stk_crud.selected_index = None
         else:
-            self.stk_crud.selected_index = int(self.buttonbar_grid.active.value)
+            self.stk_crud.selected_index = self.buttonbar_grid.active_index
 
     def _check_one_row_selected(self):
         if len(self.grid.selected_indexes) > 1:
-            raise Exception(
-                markdown("  👇 _Please only select ONLY one row from the table!_")
-            )
+            raise Exception("👇 <i>Please only select ONLY one row from the table!</i>")
 
     # edit row
     # --------------------------------------------------------------------------
@@ -457,7 +567,8 @@ class EditGrid(w.VBox, TitleDescription):
             self.grid.selections = selections
 
     def _set_ui_edit_to_selected_row(self):
-        self.ui_edit.value = self.grid.selected
+        if self.grid.selected is not None:
+            self.ui_edit.value = self.grid.selected
         self.ui_edit.savebuttonbar.unsaved_changes = False
 
     def _patch(self):
@@ -471,8 +582,8 @@ class EditGrid(w.VBox, TitleDescription):
 
         except Exception as e:
             self.buttonbar_grid.edit.value = False
-            self.buttonbar_grid.message.value = markdown(
-                "  👇 _Please select one row from the table!_ "
+            self.buttonbar_grid.message.value = (
+                "👇 <i>Please select one row from the table!</i> "
             )
             traceback.print_exc()
 
@@ -482,7 +593,7 @@ class EditGrid(w.VBox, TitleDescription):
     # --------------------------------------------------------------------------
     def _save_add_to_grid(self):
         if self.datahandler is None:
-            if not self.grid._data["data"]:  # If no data in grid
+            if len(self.grid._data["data"]) == 0:  # If no data in grid
                 self.value = tuple([self.ui_add.value])
             else:
                 # Append new row onto data frame and set to grid's data.
@@ -534,8 +645,8 @@ class EditGrid(w.VBox, TitleDescription):
     def _copy(self):
         try:
             if self.grid.selected_indexes == []:
-                self.buttonbar_grid.message.value = markdown(
-                    "  👇 _Please select a row from the table!_ "
+                self.buttonbar_grid.message.value = (
+                    "  👇 <i>Please select a row from the table!</i> "
                 )
             else:
                 if not self.show_copy_dialogue:
@@ -547,14 +658,14 @@ class EditGrid(w.VBox, TitleDescription):
                         self._copy_selected_to_end()
                         # ^ add copied values. note. above syntax required to avoid editing in place.
 
-                    self.buttonbar_grid.message.value = markdown("  📝 _Copied Data_ ")
+                    self.buttonbar_grid.message.value = "📝 <i>Copied Data</i> "
                     self.buttonbar_grid.copy.value = False
 
                 else:
                     print("need to implement show copy dialogue")
         except Exception as e:
-            self.buttonbar_grid.message.value = markdown(
-                "  👇 _Please select a row from the table!_ "
+            self.buttonbar_grid.message.value = (
+                "  👇 <i>Please select a row from the table!</i> "
             )
             traceback.print_exc()
 
@@ -564,7 +675,7 @@ class EditGrid(w.VBox, TitleDescription):
     # --------------------------------------------------------------------------
     def _reload_datahandler(self):
         self._reload_all_data()
-        self.buttonbar_grid.message.value = markdown("  🔄 _Reloaded Data_ ")
+        self.buttonbar_grid.message.value = "🔄 <i>Reloaded Data</i> "
 
     def _reload_all_data(self):
         if self.datahandler is not None:
@@ -583,7 +694,7 @@ class EditGrid(w.VBox, TitleDescription):
                 if i not in self.grid.selected_indexes
             ]
             # ^ Only set for values NOT in self.grid.selected_indexes
-        self.buttonbar_grid.message.value = markdown("  🗑️ _Deleted Row_ ")
+        self.buttonbar_grid.message.value = "🗑️ <i>Deleted Data</i> "
         if self.close_crud_dialogue_on_action:
             self.buttonbar_grid.delete.value = False
 
@@ -601,14 +712,32 @@ class EditGrid(w.VBox, TitleDescription):
                     self.ui_delete.value = self.grid.selected_dict
             else:
                 self.buttonbar_grid.delete.value = False
-                self.buttonbar_grid.message.value = markdown(
-                    "  👇 _Please select at least one row from the table!_"
+                self.buttonbar_grid.message.value = (
+                    "👇 <i>Please select at least one row from the table!</i>"
                 )
 
         except Exception as e:
             print("delete error")
             traceback.print_exc()
 
+    # io
+    # --------------------------------------------------------------------------
+    def _io(self):
+        ui = self._ensure_ui_io_initialised()
+        if ui is None:
+            self.buttonbar_grid.io.value = False
+            self.buttonbar_grid.message.value = (
+                "⚠️ <i>Import/Export unavailable for this configuration</i>"
+            )
+            return
+        try:
+            ui.value = self.value
+        except Exception as exc:
+            logger.warning("Unable to sync IO widget value: %s", exc)
+        if hasattr(ui, "traits") and "upload_status" in ui.traits():
+            ui.upload_status = "None"
+        
+
 
 # -
 
@@ -645,7 +774,7 @@ if __name__ == "__main__":
         )
 
     title = "The Wonderful Edit Grid Application"
-    description = markdown("Useful for all editing purposes whatever they may be 👍")
+    description = "Useful for all editing purposes whatever they may be 👍"
     editgrid = EditGrid(
         schema=TestDataFrame,
         title=title,
@@ -657,10 +786,96 @@ if __name__ == "__main__":
         close_crud_dialogue_on_action=False,
         global_decimal_places=1,
         column_width={"String": 400},
+        show_ui_io=True
     )
     editgrid.observe(lambda c: print("_value changed"), "_value")
     display(editgrid)
 
+if __name__ == "__main__":
+
+    class DataFrameCols(BaseModel):
+        string: str = Field(
+            "string", json_schema_extra=dict(column_width=400, section="a")
+        )
+
+    class TestDataFrame(RootModel):
+        """a description of TestDataFrame"""
+
+        root: ty.List[DataFrameCols] = Field(
+            json_schema_extra=dict(
+                format="dataframe", datagrid_index_name=("section", "title")
+            ),
+        )
+
+    editgrid.update_from_schema(TestDataFrame, value=[{"string": "Test"}] * 10)
+
+if __name__ == "__main__":
+    from pydantic import RootModel
+
+    # Test: EditGrid instance with multi-indexing.
+    AUTO_GRID_DEFAULT_VALUE = [
+        {
+            "string": None,
+            "integer": 1,
+            "floater": 3.14,
+        },
+    ]
+    AUTO_GRID_DEFAULT_VALUE = AUTO_GRID_DEFAULT_VALUE * 4
+    AUTO_GRID_DEFAULT_VALUE = AUTO_GRID_DEFAULT_VALUE + [
+        {
+            "string": None,
+            "integer": None,
+            "floater": None,
+        },
+    ]
+
+    class DataFrameCols(BaseModel):
+        string: ty.Optional[str] = Field(
+            "string",
+            json_schema_extra=dict(
+                column_width=400,
+            ),
+        )
+        integer: ty.Optional[int] = Field(
+            1,
+            json_schema_extra=dict(
+                column_width=80,
+            ),
+        )
+        floater: ty.Optional[float] = Field(
+            None,
+            json_schema_extra=dict(
+                column_width=70,
+            ),
+        )
+
+    class TestDataFrame(RootModel):
+        """a description of TestDataFrame"""
+
+        root: ty.List[DataFrameCols] = Field(
+            default=AUTO_GRID_DEFAULT_VALUE,
+            json_schema_extra=dict(
+                format="dataframe",
+            ),
+        )
+
+    title = "The Wonderful Edit Grid Application"
+    description = "Useful for all editing purposes whatever they may be 👍"
+    editgrid = EditGrid(
+        schema=TestDataFrame,
+        title=title,
+        description=description,
+        # transposed=True,
+        ui_add=None,
+        ui_edit=None,
+        warn_on_delete=True,
+        show_copy_dialogue=False,
+        close_crud_dialogue_on_action=False,
+        global_decimal_places=1,
+        column_width={"String": 400},
+    )
+    editgrid.observe(lambda c: print("_value changed"), "_value")
+    display(editgrid)
 
 if __name__ == "__main__":
     editgrid.grid.order = ("floater", "string")
@@ -678,7 +893,7 @@ if __name__ == "__main__":
         root: list[TestListCol]
 
     gr = EditGrid(Test, value=[{"li_col": ["a", "b"], "stringy": "string", "num": 23}])
-    display(gr)
+    display(gr)  # TODO: this needs fixing. not handling the list correctly. 
 
 if __name__ == "__main__":
     import random
@@ -689,9 +904,10 @@ if __name__ == "__main__":
         fn_patch=lambda v: v,
         fn_delete=lambda v: print(v),
         fn_copy=lambda v: print(v),
+        fn_io = lambda v: print("io")
     )
     title = "The Wonderful Edit Grid Application"
-    description = markdown("Useful for all editing purposes whatever they may be 👍")
+    description = "Useful for all editing purposes whatever they may be 👍"
     editgrid = EditGrid(
         schema=TestDataFrame,
         title=title,
@@ -700,10 +916,6 @@ if __name__ == "__main__":
     )
     display(editgrid)
 
-# +
-# editgrid._reload_datahandler()
-# -
-
 if __name__ == "__main__":
 
     class DataFrameCols(BaseModel):
@@ -726,17 +938,19 @@ if __name__ == "__main__":
                     floater=2.5,
                 ).model_dump()
             ],
-            json_schema_extra=dict(format="dataframe"),
-            # datagrid_index_name=("section", "title"),
+            json_schema_extra=dict(
+                format="dataframe", datagrid_index_name=("section", "title")
+            ),
         )
 
-    description = markdown(
+    description = (
         "<b>The Wonderful Edit Grid Application</b><br>Useful for all editing purposes"
         " whatever they may be 👍"
     )
     editgrid = EditGrid(
         schema=TestDataFrame,
         description=description,
+        # transposed=True,
         ui_add=None,
         ui_edit=None,
         warn_on_delete=True,
