@@ -1,21 +1,5 @@
-# -*- coding: utf-8 -*-
-# ---
-# jupyter:
-#   jupytext:
-#     text_representation:
-#       extension: .py
-#       format_name: light
-#       format_version: '1.5'
-#       jupytext_version: 1.14.0
-#   kernelspec:
-#     display_name: Python 3 (ipykernel)
-#     language: python
-#     name: python3
-# ---
-
 # +
-# %run ../_dev_maplocal_params.py
-# %load_ext lab_black
+
 
 from IPython.display import display
 from ipyautoui.basemodel import BaseModel
@@ -269,8 +253,8 @@ if __name__ == "__main__":
     c1_str_not_exists = (
         "📁⚠️ - `{}` : folder does not exist in location. It will be created on load"
     )
-    c1 = (
-        lambda value: c1_str_exists.format(make_new_path(value["fdir"]))
+    c1 = lambda value: (
+        c1_str_exists.format(make_new_path(value["fdir"]))
         if value["fdir"].is_dir()
         else c1_str_not_exists.format(make_new_path(value["fdir"]))
     )
