@@ -12,17 +12,12 @@ class ComplexSerialisation(BaseModel):
     file_chooser: pathlib.Path = pathlib.Path(".")
     file_upload_to_dir: pathlib.Path = Field(
         description="uploaded files automatically get dumped in cwd or fdir if defined for the AutoUi",
-        json_schema_extra=dict(autoui="ipyautoui.custom.FileUploadToDir"))
-    date_picker_string: ty.Optional[
-        date
-    ] = (
-        date.today()
+        json_schema_extra=dict(autoui="ipyautoui.custom.FileUploadToDir"),
     )
-    naive_datetime_picker_string: ty.Optional[
-        datetime
-    ] = datetime.now()
+    date_picker_string: ty.Optional[date] = date.today()
+    naive_datetime_picker_string: ty.Optional[datetime] = datetime.now()
     color_picker_ipywidgets: Color = "#f5f595"
     markdown_widget: str = Field(json_schema_extra=dict(format="markdown"))
-    any_of: ty.Union[conint(ge=0, le=3), str] = Field(
+    any_of: ty.Union[conint(ge=0, le=3), str] = Field( # type: ignore
         description="anyof widget. once selected the type is fixed."
     )
