@@ -1,13 +1,12 @@
 """
 An example schema definition that demonstrates the current capability of the AutoUi class
 """
+
 import typing as ty
 import annotated_types
 from enum import Enum
 from pydantic import StringConstraints, Field, conint, confloat, ConfigDict
 from ipyautoui.basemodel import BaseModel
-import pandas as pd
-from pathlib import PurePosixPath
 from typing_extensions import Annotated
 from enum import IntEnum
 
@@ -25,7 +24,8 @@ class Number(IntEnum):
     ONE = 1
     TWO = 2
     THREE = 3
-    
+
+
 class LongEnum(str, Enum):
     a = "a"
     b = "b"
@@ -63,6 +63,7 @@ class CoreIpywidgets(BaseModel):
     )
     float_slider: float = Field(default=2.2, ge=1, le=3)
     float_text: float = 2.2
+    float_text_with_unit: float = Field(default=2.2, json_schema_extra=dict(unit="mm"))
     float_text_locked: float = Field(default=2.2, json_schema_extra=dict(disabled=True))
     float_range_slider: tuple[confloat(ge=0, le=4), confloat(ge=0, le=4)] = Field(
         default=(0, 2.2)
@@ -80,17 +81,27 @@ class CoreIpywidgets(BaseModel):
         default=FruitEnum.apple,
         description="updated description",
     )
-    dropdown_simple: str = Field(default="asd", json_schema_extra=dict(enum=["asd", "asdf"]))
+    dropdown_simple: str = Field(
+        default="asd", json_schema_extra=dict(enum=["asd", "asdf"])
+    )
     text: str = Field(default="adsf", description="a description about my string")
-    text_short: Annotated[
-        str, StringConstraints(min_length=0, max_length=20)
-    ] = "short text"
+    text_short: Annotated[str, StringConstraints(min_length=0, max_length=20)] = (
+        "short text"
+    )
     textarea: Annotated[str, StringConstraints(min_length=0, max_length=800)] = Field(
         "long text " * 50, description="long text field"
     )
-    select_multiple: ty.List[FruitEnum] = Field((FruitEnum.apple,), description="a short select multiple list")
-    select_multiple_nullable: ty.Optional[ty.List[FruitEnum]] = Field(None, description="a short nullable select multiple list")
-    tags_input: ty.List[LongEnum] = Field((LongEnum.a,), description="a long select multiple list")
-    tags_input_nullable: ty.Optional[ty.List[LongEnum]] = Field((LongEnum.a,), description="a long nullable select multiple list")
+    select_multiple: ty.List[FruitEnum] = Field(
+        (FruitEnum.apple,), description="a short select multiple list"
+    )
+    select_multiple_nullable: ty.Optional[ty.List[FruitEnum]] = Field(
+        None, description="a short nullable select multiple list"
+    )
+    tags_input: ty.List[LongEnum] = Field(
+        (LongEnum.a,), description="a long select multiple list"
+    )
+    tags_input_nullable: ty.Optional[ty.List[LongEnum]] = Field(
+        (LongEnum.a,), description="a long nullable select multiple list"
+    )
 
     model_config = ConfigDict(json_schema_extra=dict(show_raw=True))
