@@ -17,18 +17,16 @@ DATAGRID_TEST_VALUE = [
 
 
 class DataFrameCols(BaseModel):
-    string: str = Field("string", column_width=200)
+    string: str = Field("string", json_schema_extra=dict(column_width=200))
     integer: int = Field(1)
-    floater: float = Field(3.1415, column_width=70)
-    something_else: float = Field(324, column_width=100)
+    floater: float = Field(3.1415, json_schema_extra=dict(column_width=70))
+    something_else: float = Field(324, json_schema_extra=dict(column_width=100))
 
 
 class NestedEditableGrid(BaseModel):
     title: str = "My editable Dataframe"
-    reference: str = "some ref data"
+    reference: ty.Optional[str] = None  # TODO: this creates formatting issue
     grid: ty.List[DataFrameCols] = Field(
         default=DATAGRID_TEST_VALUE,
-        # default_factory=lambda: DATAGRID_TEST_VALUE, # TODO: AutoUi isn't getting data when set using default_factory. make this work!
-        format="DataFrame",
-        global_decimal_places=2,
+        json_schema_extra=dict(format="DataFrame", global_decimal_places=2),
     )
