@@ -1,12 +1,15 @@
-from pydantic import Field, conint, constr
+from pydantic import Field, constr, RootModel
 from ipyautoui.basemodel import BaseModel
+from typing_extensions import Annotated
+
 
 class NestedObject(BaseModel):
     """description in docstring"""
 
     string1: str = Field(default="adsf", description="a description about my string")
-    int_slider1: conint(ge=0, le=3) = 2
+    int_slider1: Annotated[int, Field(ge=0, le=3)] = 2
     int_text1: int = 1
 
-class RootArray(BaseModel):
-    __root__: list[NestedObject]
\ No newline at end of file
+
+class RootArray(RootModel):
+    root: list[NestedObject]
